//**********************
// I2C Function
// 
// (C)Copyright 2010 All rights reserved by Y.Onodera
// http://einstlab.web.fc2.com
//**********************
#if defined(__dsPIC30F__)
#include <p30fxxxx.h>
#elif defined(__dsPIC33F__)
#include <p33Fxxxx.h>
#elif defined(__PIC24H__)
#include <p24Hxxxx.h>
#elif defined(__PIC24F__)
#include <p24Fxxxx.h>
#endif
#include <i2c.h>


//**********************
// FOSC, PLL, FCY
//**********************
#if defined(__dsPIC30F__)
#define	FOSC	7372800UL
#define	PLL	16
#define	FCY	FOSC*PLL/4
#else
#define	FOSC	8000000UL
#define	PLL	1
#define	FCY	FOSC*PLL/2
#endif


//**********************
// BaudRate
//**********************
//#define FSCL 100000
#define FSCL 400000
//#define FSCL 1000000


unsigned int _I2CBRG;
unsigned int _I2CCON;


void InitI2C(void)
{

	// if slave
	I2CADD = 0xA0;

	// Trun off I2C
	CloseI2C();
	// Configure I2C interrupt
	ConfigIntI2C(	MI2C_INT_OFF &
					SI2C_INT_OFF );
#if defined(__PIC24F__)
	// Configure I2C
	// I2CBRG=((Fcy/Fscl)-Fcy/10000000)-1
	//      =((4000000/100000)-4000000/10000000)-1
	//      =39
	// Fscl=Fcy/(I2CBRG+1+Fcy/1111111)
	//         =4000000/(267+1+4000000/10000000)
	//         =99009
	// Error=(99009-100000)/100000
	//      =-0.99%
	_I2CBRG=((FCY/FSCL)-FCY/10000000)-1+0.5;
#else
	// Configure I2C
	// I2CBRG=((Fcy/Fscl)-Fcy/1111111)-1
	//      =((29491200/100000)-29491200/1111111)-1
	//      =267
	// Fscl=Fcy/(I2CBRG+1+Fcy/1111111)
	//         =29491200/(267+1+29491200/1111111)
	//         =100125
	// Error=(100125-100000)/100000
	//      =0.13%
	_I2CBRG=((FCY/FSCL)-FCY/1111111)-1+0.5;
#endif


	_I2CCON=

		I2C_ON &
//		I2C_OFF &

//		I2C_IDLE_STOP &
		I2C_IDLE_CON &

//		I2C_CLK_REL &
		I2C_CLK_HLD &

//		I2C_IPMI_EN &
		I2C_IPMI_DIS &

//		I2C_10BIT_ADD &
		I2C_7BIT_ADD &

//		I2C_SLW_DIS &
		I2C_SLW_EN &	// if 400KHz

//		I2C_SM_EN &
		I2C_SM_DIS &

//		I2C_GCALL_EN &
		I2C_GCALL_DIS &

//		I2C_STR_EN &
		I2C_STR_DIS &

//		I2C_ACK &
		I2C_NACK &

//		I2C_ACK_EN &
		I2C_ACK_DIS &

//		I2C_RCV_EN &
		I2C_RCV_DIS &

//		I2C_STOP_EN &
		I2C_STOP_DIS &

//		I2C_RESTART_EN &
		I2C_RESTART_DIS &

//		I2C_START_EN;
		I2C_START_DIS;


	// Open I2C
	OpenI2C(_I2CCON, _I2CBRG);

}


unsigned char EEPROMreadI2C(unsigned int I2Cadr, unsigned int EEPadr)
{
	unsigned char data;

	IdleI2C();
	StartI2C();
	while(I2CCONbits.SEN);

	// set I2C address with writing
	MasterWriteI2C(I2Cadr & 0xFE);
	while(I2CSTATbits.TBF);
	while(I2CSTATbits.ACKSTAT);

	// write EEPROM high address
	MasterWriteI2C(EEPadr>>8);
	while(I2CSTATbits.TBF);
	while(I2CSTATbits.ACKSTAT);

	// write EEPROM low address
	MasterWriteI2C(EEPadr & 0xFF);
	while(I2CSTATbits.TBF);
	while(I2CSTATbits.ACKSTAT);

	RestartI2C();
	while(I2CCONbits.RSEN);

	// set I2C address with reading
	MasterWriteI2C(I2Cadr | 0x01);
	while(I2CSTATbits.TBF);
	while(I2CSTATbits.ACKSTAT);

	// read data
	data = MasterReadI2C();
	NotAckI2C();
	while(I2CCONbits.ACKEN);

	StopI2C();
	while(I2CCONbits.PEN);

	return data;
}


void EEPROMwriteI2C(unsigned int I2Cadr, unsigned int EEPadr, unsigned char data)
{

	StartI2C();
	while(I2CCONbits.SEN);

	// set I2C address with writing
	MasterWriteI2C(I2Cadr & 0xFE);
	while(I2CSTATbits.TBF);
	while(I2CSTATbits.ACKSTAT);

	// write EEPROM high address
	MasterWriteI2C(EEPadr>>8);
	while(I2CSTATbits.TBF);
	while(I2CSTATbits.ACKSTAT);

	// write EEPROM low address
	MasterWriteI2C(EEPadr & 0xFF);
	while(I2CSTATbits.TBF);
	while(I2CSTATbits.ACKSTAT);

	// write EEPROM data
	MasterWriteI2C(data);
	while(I2CSTATbits.TBF);
	while(I2CSTATbits.ACKSTAT);

	StopI2C();
	while(I2CCONbits.PEN);

}

